// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "LMSpawnPosition.h"
#include "LMWaveStructure.h"
#include "LMWaveManager.generated.h"

class ALMEnemy;

UCLASS()
class LEGUMEMIX_API ALMWaveManager : public AActor
{
	GENERATED_BODY()
	
public:	
	// Sets default values for this actor's properties
	ALMWaveManager();

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, meta = (AllowPrivateAccess = "true"), Category = "Legumix")
	FString CurrentWaveName; // Name of the current wave
	
	UPROPERTY(EditAnywhere, BlueprintReadWrite, meta = (AllowPrivateAccess = "true"), Category = "Legumix")
	TArray<ALMSpawnPosition*> SpawnPositionsList; // Liste des ennemis
	
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, meta = (AllowPrivateAccess = "true"), Category = "Legumix")
	TArray<ALMEnemy*> EnemyAliveList; // Liste des ennemis

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, meta = (AllowPrivateAccess = "true"), Category = "Legumix")
	int EnemyNumberInWave; // Nombre d'ennemis dans la wave

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, meta = (AllowPrivateAccess = "true"), Category = "Legumix")
	int MaxEnemyInstantiate; // Nombre d'ennemis max spawnés

	UPROPERTY(EditAnywhere, BlueprintReadWrite, meta = (AllowPrivateAccess = "true"), Category = "Legumix")
	float BreakTime; // Nombre d'ennemis dans la wave

	UPROPERTY(EditAnywhere, BlueprintReadWrite, meta = (AllowPrivateAccess = "true"), Category = "Legumix")
	UDataTable* WaveDatePreset;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category="Legumix")
	TArray<TSubclassOf<ALMEnemy>> AllEnemyType;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category="Legumix")
	int EnemySpawned = 0;
	
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category="Legumix")
	TArray<int> EnemiesPerType;
	
protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;
	virtual void StartWave();
	virtual void SpawnEnemy(ALMSpawnPosition* spawnPosition);
	UFUNCTION()
	virtual void EnemyDead(ALMEnemy* enemyToRemoveFromLife);
	virtual bool RemainsEnemyToSpawn();
	virtual void CheckForSpawnerOK();
	virtual int TotalOfEnemyToStillSpawn();

	virtual void GetRandomDataWaveRow();

public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

private:
	TArray<ALMSpawnPosition*> SpawnPositionsOK;
	
	FTimerHandle BreakTimer;
	bool OnCooldown;
};
